from torchvision.datasets import VisionDataset
from PIL import Image
import os
import os.path
from pyvww.utils import VisualWakeWords


class VisualWakeWordsClassification(VisionDataset):
    """`Visual Wake Words <https://arxiv.org/abs/1906.05721>`_ Dataset.
    Args:
        root (string): Root directory where COCO images are downloaded to.
        annFile (string): Path to json visual wake words annotation file.
        transform (callable, optional): A function/transform that  takes in an PIL image
            and returns a transformed version. E.g, ``transforms.ToTensor``
        target_transform (callable, optional): A function/transform that takes in the
            target and transforms it.
    """
    def __init__(self, root, annFile, transform=None, target_transform=None, transforms=None):
        super(VisualWakeWordsClassification, self).__init__(root, transforms, transform, target_transform)
        self.vww = VisualWakeWords(annFile)
        self.ids = list(sorted(self.vww.imgs.keys()))

    def __getitem__(self, index):
        """
        Args:
            index (int): Index
        Returns:
            tuple: Tuple (image, target). target is the index of the target class.
        """
        vww = self.vww
        img_id = self.ids[index]
        ann_ids = vww.getAnnIds(imgIds=img_id)
        if ann_ids:
            target = vww.loadAnns(ann_ids)[0]['category_id']
        else:
            target = 0

        path = vww.loadImgs(img_id)[0]['file_name']

        img = Image.open(os.path.join(self.root, path)).convert('RGB')
        if self.transform is not None:
            img = self.transform(img)

        if self.target_transform is not None:
            target = self.target_transform(target)

        return img, target

    def __len__(self):
        return len(self.ids)



import os
import io
from setuptools import find_packages, setup


def read(*names, **kwargs):
    with io.open(os.path.join(os.path.dirname(__file__), *names),
                 encoding=kwargs.get("encoding", "utf8")) as fp:
        return fp.read()


readme = read('README.md')

setup(
    name='pyvww',
    packages=find_packages(),
    version='0.1.1',
    description='Python API to work with the Visual Wake Words Dataset.',
    long_description=readme,
    long_description_content_type="text/markdown",
    author='Maxim Bonnaerens',
    author_email="maxim@bonnaerens.be",
    url='https://github.com/Mxbonn/visualwakewords',
    license='Apache 2.0',
    install_requires=[
        'pycocotools',
        'torchvision'
        ]
)



import matplotlib.patches as patches
import matplotlib.pyplot as plt
import numpy as np
from pycocotools.coco import COCO


class VisualWakeWords(COCO):
    def __init__(self, *args):
        super(VisualWakeWords, self).__init__(*args)

    def showAnns(self, anns):
        """
        Display the specified annotations.
        :param anns (array of object): annotations to display
        :return: None
        """
        if len(anns) == 0:
            return 0
        ax = plt.gca()
        ax.set_autoscale_on(False)
        for ann in anns:
            c = (np.random.random((1, 3)) * 0.6 + 0.4).tolist()[0]
            if ann['category_id'] == 1:
                [x, y, width, height] = ann['bbox']
                rect = patches.Rectangle((x, y), width, height, edgecolor=c, facecolor=c, linewidth=2, alpha=0.4)
                ax.add_patch(rect)

    def download(self, *args):
        raise AttributeError("Cannot download Visual Wake Words Dataset. "
                             "See instructions on github.com/Mxbonn/visualwakewords to create"
                             "the Visual Wake Words Dataset.")

    def loadRes(self, resFile):
        raise AttributeError("Method not implemented for the Visual Wake Words Dataset.")

    def annToRLE(self, ann):
        raise AttributeError("Method not implemented for the Visual Wake Words Dataset.")

    def annToMask(self, ann):
        raise AttributeError("Method not implemented for the Visual Wake Words Dataset.")

